import json
import os
import tempfile

import responses
import requests

from siskin.utils import (URLCache, dictcheck, get_task_import_cache, nwise,
                          random_string, scrape_html_listing, SetEncoder)

def test_set_encoder_dumps():
	assert json.dumps({'x': {0, 1, 2}}, cls=SetEncoder) == '{"x": [0, 1, 2]}'

def test_dictcheck():
    assert dictcheck({"name": "x"}, contains=["name"]) is True
    assert dictcheck({"name": "x"}, contains=["name"], absent=["somekey"]) is True
    assert dictcheck({"name": "x", "somekey": 123},
                     contains=["name"], absent=["somekey"]) is False
    assert dictcheck({"somekey": None}, absent=["somekey"]) is True
    assert dictcheck({}, absent=["somekey"]) is True


def test_nwise():
    assert list(nwise(range(4))) == [(0, 1), (2, 3)]
    assert list(nwise(range(4), n=2)) == [(0, 1), (2, 3)]
    assert list(nwise(range(4), n=3)) == [(0, 1, 2), (3,)]
    assert list(nwise(range(4), n=10)) == [(0, 1, 2, 3)]
    assert list(nwise([], n=10)) == []

def test_random_string():
    assert len(random_string()) == 16
    assert len(random_string(length=1000)) == 1000
    assert len(random_string(length=-1)) == 0
    assert random_string(length=-100) == ''
    for char in ' \t\n.:,;#~+-?=[]()/&%$"!':
        assert char not in random_string()

def test_get_task_import_cache():
    mapping, path = get_task_import_cache()
    assert os.path.exists(path)
    assert isinstance(mapping, dict)

def test_get_cache_file(tmpdir):
    cache = URLCache(directory=str(tmpdir))
    fn = cache.get_cache_file("http://x.com")
    assert fn.startswith(str(tmpdir))

@responses.activate
def test_scrape_html_listing():
    responses.add(responses.GET, 'http://fake.com/1',
                  body='<html></html>', status=200)
    resp = requests.get('http://fake.com/1')
    assert scrape_html_listing('http://fake.com/1') == []

    body = """
    <!doctype html><title>Index of /archlinux/iso/2018.12.01</title><h1>Index
    of /archlinux/iso/2018.12.01</h1><table><tr><th valign=top><img
    src=/icons/blank.gif alt=[ICO]><th><a href="?C=N;O=D">Name</a><th><a
    href="?C=M;O=A">Last modified</a><th><a href="?C=S;O=A">Size</a><th><a
    href="?C=D;O=A">Description</a><tr><th colspan=5><hr><tr><td valign=top><img
    src=/icons/back.gif alt=[PARENTDIR]><td><a href=/archlinux/iso/>Parent
    Directory</a><td>&nbsp;<td align=right>-<td>&nbsp;<tr><td valign=top><img
    src=/icons/folder.gif alt=[DIR]><td><a href=arch/>arch/</a><td
    align=right>2018-12-01 11:16<td align=right>-<td>&nbsp;<tr><td valign=top><img
    src=/icons/unknown.gif alt="[   ]"><td><a
    href=archlinux-2018.12.01-x86_64.iso>archlinux-2018.12.01-x86_64.iso</a><td
    align=right>2018-12-01 11:17<td align=right>588M<td>&nbsp;<tr><td
    valign=top><img src=/icons/unknown.gif alt="[   ]"><td><a
    href=archlinux-2018.12.01-x86_64.iso.sig>archlinux-2018.12.01-x86_64.iso.sig</a><td
    align=right>2018-12-01 11:18<td align=right>310<td>&nbsp;<tr><td
    valign=top><img src=/icons/unknown.gif alt="[   ]"><td><a
    href=archlinux-2018.12.01-x86_64.iso.torrent>archlinux-2018.12.01-x86_64.iso.torrent</a><td
    align=right>2018-12-01 11:18<td align=right>38K<td>&nbsp;<tr><td
    valign=top><img src=/icons/compressed.gif alt="[   ]"><td><a
    href=archlinux-bootstrap-2018.12.01-x86_64.tar.gz>archlinux-bootstrap-2018.12.01-x86_64.tar.gz</a><td
    align=right>2018-12-01 11:18<td align=right>141M<td>&nbsp;<tr><td
    valign=top><img src=/icons/unknown.gif alt="[   ]"><td><a
    href=archlinux-bootstrap-2018.12.01-x86_64.tar.gz.sig>archlinux-bootstrap-2018.12.01-x86_64.tar.gz.sig</a><td
    align=right>2018-12-01 11:18<td align=right>310<td>&nbsp;<tr><td
    valign=top><img src=/icons/text.gif alt=[TXT]><td><a
    href=md5sums.txt>md5sums.txt</a><td align=right>2018-12-01 11:18<td
    align=right>145<td>&nbsp;<tr><td valign=top><img src=/icons/text.gif
    alt=[TXT]><td><a href=sha1sums.txt>sha1sums.txt</a><td align=right>2018-12-01
    11:18<td align=right>161<td>&nbsp;<tr><th
    colspan=5><hr></table><address>Apache/2.4.25 (Debian) Server at
    ftp.halifax.rwth-aachen.de Port 80</address>
    """

    expected = [
        'http://fake.com/archlinux-2018.12.01-x86_64.iso',
        'http://fake.com/archlinux-2018.12.01-x86_64.iso.sig',
        'http://fake.com/archlinux-2018.12.01-x86_64.iso.torr',
        'http://fake.com/archlinux-bootstrap-2018.12.01-x86_64.tar.gz',
        'http://fake.com/archlinux-bootstrap-2018.12.01-x86_64.tar.gz.sig',
        'http://fake.com/md5sums.txt',
        'http://fake.com/sha1sums.txt',
    ]

    responses.add(responses.GET, 'http://fake.com/1',
                  body=body, status=200)
    resp = requests.get('http://fake.com/1')
    assert scrape_html_listing('http://fake.com/1') == expected




#!/usr/bin/env python3
# coding: utf-8

"""
Issues: refs #7462, #12737, #13168, #13234, #13261.

This source is manually profiled for ADLR.
"""

from __future__ import print_function

import io
import re
import sys

import xmltodict
import marcx

relevant_sets = ["com_community_1080400", "com_community_10800", "col_collection_1080401", "col_collection_1080402", "col_collection_1080403",
                 "col_collection_1080404", "col_collection_1080405", "col_collection_1080406", "col_collection_1080407", "col_collection_1080408",
                 "col_collection_1080409", "col_collection_1080410", "col_collection_1080411", "col_collection_1080412", "col_collection_10800",
                 "col_collection_1080400", "col_collection_10899"]

collection_map = {
    "com_community_1080400": u"Massenkommunikation",
    "com_community_10800": u"Kommunikationswissenschaften",
    "col_collection_1080401": u"Rundfunk, Telekommunikation",
    "col_collection_1080402": u"Druckmedien",
    "col_collection_1080403": u"Andere Medien",
    "col_collection_1080404": u"Interaktive, elektronische Medien",
    "col_collection_1080405": u"Medieninhalte, Aussagenforschung",
    "col_collection_1080406": u"Kommunikatorforschung, Journalismus",
    "col_collection_1080407": u"Wirkungsforschung, Rezipientenforschung",
    "col_collection_1080408": u"Meinungsforschung",
    "col_collection_1080409": u"Werbung, Public Relations, Öffentlichkeitsarbeit",
    "col_collection_1080410": u"Medienpädagogik",
    "col_collection_1080411": u"Medienpolitik, Informationspolitik, Medienrecht",
    "col_collection_1080412": u"Medienökonomie, Medientechnik",
    "col_collection_10800": u"Kommunikationswissenschaften",
    "col_collection_1080400": u"Massenkommunikation (allgemein)",
    "col_collection_10899": u"Sonstiges zu Kommunikationswissenschaften"
}


def get_field(tag):
    try:
        return xmlrecord["metadata"]["oai_dc:dc"][tag]
    except:
        return ""


inputfilename = "30_input.xml"
outputfilename = "30_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")

xmlfile = inputfile.read()
xmlrecords = xmltodict.parse(xmlfile)

for xmlrecord in xmlrecords["Records"]["Record"]:

    marcrecord = marcx.Record(to_unicode=True, force_utf8=True)

    # Format
    format = get_field("dc:type")
    if "Monographie" in format or "Sammelwerk" in format or "Konferenzband" in format:
        leader = "     cam  22        4500"
        f935b = "cofz"
        f935c = ""
    if "Sammelwerksbeitrag" in format:
        leader = "     naa  22        4500"
        f935b = "cofz"
        f935c = ""
    elif "Diplomarbeit" in format or "Habilitationsschrift" in format or "Magisterarbeit" in format or "Dissertation" in format:
        leader = "     cam  22        4500"
        f935b = "cofz"
        f935c = "hs"
    else:
        leader = "     naa  22        4500"
        f935b = "cofz"
        f935c = ""

    # Leader
    marcrecord.leader = leader

    # Identifier
    f001 = xmlrecord["header"]["identifier"]
    regexp = re.match("oai:gesis\.izsoz\.de:document\/(\d+)", f001)
    if regexp:
        f001 = regexp.group(1)
        marcrecord.add("001", data="finc-30-" + f001)
    else:
        print("Fehler! Der Identifikator konnte nicht extrahiert werden: " + f001)

    # 007
    marcrecord.add("007", data="cr")

    marcrecord.strict = False

    # ISBN
    identifiers = get_field("dc:identifier")
    for identifier in identifiers:
        if isinstance(identifier, str):
            regexp = re.search("^(978-.*)", identifier)
            if regexp:
                f020a = regexp.group(1)
                marcrecord.add("020", a=f020a)
                break

    # ISSN
    identifiers = get_field("dc:identifier")
    for identifier in identifiers:
        if identifier is None:
            print('skipping None id', file=sys.stderr)
            continue
        regexp = re.search("^(\d\d\d\d-\d\d\d\d)", identifier)
        if regexp:
            f022a = regexp.group(1)
            marcrecord.add("022", a=f022a)
            break

    # Sprache
    """ Ist nur zweimal enthalten bei über 45.000 Titeln und einmal davon fehlerhaft, deshalb wird sie hier komplett weggelassen. """

    # 1. geistiger Schöpfer
    authors = get_field("dc:creator")
    if isinstance(authors, list):
        marcrecord.add("100", a=authors[0], e="aut")
    else:
        marcrecord.add("100", a=authors, e="aut")

    # Haupttitel
    f245 = get_field("dc:title")
    # Muss mit rein, weil an einer Stelle ein falscher, zusätzlicher title-Tag vorkommt.
    if isinstance(f245, list):
        f245 = f245[0]
    f245 = f245.split(" : ")
    if len(f245) > 1:
        f245a = f245[0]
        f245b = f245[1:]
    else:
        f245a = f245[0]
        f245b = ""
    marcrecord.add("245", a=f245a, b=f245b)

    # Erscheinungsvermerk
    # Die Angaben im Feld publisher sind sehr inkonsistent in Inhalt, Anzahl und Reihenfolge.
    # Relativ sicher ist bloß: wenn das Feld viermal vorhanden ist, steht im ersten der Verlag und im letzten der Ort.
    # Und wenn das Feld dreimal vorhanden ist im letzten der Ort
    # In den anderen Fällen ist es so sehr Kraut und Rüben, dass hier bloß das Erscheinungsjahr angegeben und der Rest weggelassen wird.
    # Andenfalls fällt die Datenqualität zu sehr ab.
    f260 = get_field("dc:publisher")
    if isinstance(f260, list) and len(f260) == 4:
        f260a = f260[3]
        f260b = f260[0]
    elif isinstance(f260, list) and len(f260) == 3:
        f260a = f260[2]
        f260b = ""
    else:
        f260a = ""
        f260b = ""
    dates = get_field("dc:date")
    for date in dates:
        regexp = re.search("^(\d\d\d\d)$", date)
        if regexp:
            f260c = regexp.group(1)
            break
        else:
            f260c = ""
    if f260a != "" and f260b != "":
        f260a = f260a + " : "
    if (f260a != "" or f260b != "") and f260c != "":
        f260c = ", " + f260c
    publisher = ["a", f260a, "b", f260b, "c", f260c]
    marcrecord.add("260", subfields=publisher)

    # Reihen / Kollektion
    collections = xmlrecord["header"]["setSpec"]
    for collection in collections:
        f490a = collection_map.get(collection, "")
        marcrecord.add("490", a=f490a)

    # Rechte
    f500a = get_field("dc:rights")
    if isinstance(f500a, list):
        marcrecord.add("500", a="Rechtehinweis: " + f500a[0])
    else:
        marcrecord.add("500", a="Rechtehinweis: " + f500a)

    # Abstract
    f520a = get_field("dc:description")
    marcrecord.add("520", a=f520a)

    # Schlagwörter
    unique_subjects = []
    subjects = get_field("dc:subject")
    for subject in subjects:
        if subject:
            # damit werden die englischen Schlagwörter weitestgehend herausgefiltert
            if subject[0].isupper():
                if subject not in unique_subjects:  # weil die Schlagwörter sich teilweise wiederholen
                    marcrecord.add("650", a=subject)
                    unique_subjects.append(subject)
            else:
                if " " in subject:  # für solche Fälle: soziale Lage, internationaler Vergleich
                    s = subject.split(" ")
                    x = s[1]
                    if x.isupper():
                        if subject not in unique_subjects:  # weil die Schlagwörter sich teilweise wiederholen
                            marcrecord.add("650", a=subject)
                            unique_subjects.append(subject)

    # weitere geistige Schöpfer
    authors = get_field("dc:creator")
    if isinstance(authors, list):
        for author in authors[1:]:
            marcrecord.add("700", a=author, e="aut")

    # Mitwirkende
    contributors = get_field("dc:contributor")
    if isinstance(contributors, list):
        for contributor in contributors:
            marcrecord.add("700", a=contributor, e="ctb")

    # übergeordnetes Werk
    journal = ""
    volume = ""
    issue = ""
    pages = ""
    sources = get_field("dc:source")
    if "Zeitschriftenartikel" in format and len(sources) == 4:
        journal = sources[0]
        volume = sources[1]
        issue = sources[2]
        pages = sources[3]
        f773g = u"%s(%s)%s, S. %s" % (volume, f260c, issue, pages)
        marcrecord.add("773", a=journal, g=f773g)
    elif sources != "":
        for source in sources:
            regexp = re.search("^(\D+)", source)
            if regexp:
                journal = regexp.group(1)
                marcrecord.add("773", a=journal)
                break

    # Link zu Datensatz und Ressource
    identifiers = get_field("dc:identifier")
    urn = False
    ssoar = ""
    for identifier in identifiers:
        if identifier:  # Martin fragen, warum hier sonst "argument of type 'NoneType' is not iterable" kommt

            if "http://www.ssoar" in identifier:  # ist manchmal ausschließlich in der Form angegeben
                ssoar = identifier
            elif "urn:nbn" in identifier:
                marcrecord.add("856", q="text/html", _3="Link zum Datensatz",
                               u="http://nbn-resolving.de/" + identifier)
                # damit bevorzugt der Permalink angezeigt wird und nicht der Link auf http://www.ssoar ...
                urn = True
            elif ".pdf" in identifier:
                marcrecord.add("856", q="text/html",
                               _3="Link zur Ressource", u=identifier)

    if urn == False:
        # Muss sein, damit der Link zum Datensatz nur einmal angezeigt wird
        marcrecord.add("856", q="text/html", _3="Link zum Datensatz", u=ssoar)

    # Medientyp
    marcrecord.add("935", b=f935b, c=f935c)

    # Kollektion
    collection_with_adlr = ["a", f001, "b", "30", "c",
                            "SSOAR Social Science Open Access Repository", "c", "ssoarAdlr"]
    collection_without_adlr = ["a", f001, "b", "30",
                               "c", "SSOAR Social Science Open Access Repository"]
    collections = xmlrecord["header"]["setSpec"]
    for relevant_set in relevant_sets:
        if relevant_set in collections:
            marcrecord.add("980", subfields=collection_with_adlr)
            break
    else:
        marcrecord.add("980", subfields=collection_without_adlr)

    outputfile.write(marcrecord.as_marc())

inputfile.close()
outputfile.close()



# coding: utf-8
# pylint: disable=C0301,E1101

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Datenbank Internetquellen, #5374, #9913, #11005.

Fixed source from the defunct http://www.medien-buehne-film.de/.

About 214M in 163 XML files, no schema, root XML element is called DBClear,
currently the files are named monday.xml.part133, monday.xml.part134, and so
on.

Configuration
-------------

[dbinet]

directory = /path/to/dir/containing/raw/files

"""

import glob
import json
import os
import tempfile

import luigi
import requests
from luigi.format import Gzip

from gluish.format import TSV
from gluish.utils import shellout
from siskin.task import DefaultTask


class DBInetTask(DefaultTask):
    """
    Base task, refs #5374, #9913, #11005, #13653.
    """
    TAG = "80"


class DBInetFiles(DBInetTask):
    """
    A list of raw files.
    """

    def run(self):
        directory = self.config.get('dbinet', 'directory')
        with self.output().open('w') as output:
            for path in sorted(glob.glob(os.path.join(directory, "*xml.part*"))):
                output.write_tsv(path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='filelist'), format=TSV)


class DBInetJSON(DBInetTask):
    """
    Combine, prepare, requires xlstproc, sed, jq.
    """

    def requires(self):
        return DBInetFiles()

    def run(self):
        _, stopover = tempfile.mkstemp(prefix='siskin-')
        with self.input().open() as handle:
            for row in handle.iter_tsv(cols=('path',)):
                shellout("""python {xmltojson} <(xsltproc {xsl} <(sed -e 's/&#.;//g' -e 's/&#..;//g' {input})) /dev/stdout |
                            jq -cr ".list[][]" >> {output} """,
                         xmltojson=self.assets('80/xmltojson.py'),
                         xsl=self.assets('80/convert.xsl'), input=row.path, output=stopover)

        luigi.LocalTarget(stopover).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj'))


class DBInetIntermediateSchema(DBInetTask):
    """
    Convert via jq. With --urlcheck, filter out dead links (takes a while). For
    faster URL checks, try: taskcat DBInetIntermediateSchema | jq -cr .url[] |
    clinker -w 200, see also: https://git.io/fAC27.
    """

    urlcheck = luigi.BoolParameter(default=False, significant=False)

    def requires(self):
        return DBInetJSON()

    def run(self):
        """
        When URLs are checked, only write records with at least one reachable URL.
        """
        output = shellout("jq -rc -f {filter} {input} | pigz -c > {output}",
                          filter=self.assets('80/filter.jq'), input=self.input().path)

        if not self.urlcheck:
            luigi.LocalTarget(output).move(self.output().path)
            return

        with luigi.LocalTarget(output).open() as handle:
            with self.output().open('w') as output:
                for i, line in enumerate(handle):
                    doc = json.loads(line)
                    okurls = []
                    for url in doc.get("url"):
                        try:
                            # Prevent MissingSchema errors.
                            if not url.startswith("http"):
                                url = "http://%s" % url

                            resp = requests.get(url, timeout=20)
                            self.logger.debug("[%s] #%d %s", resp.status_code, i, url)
                            if resp.status_code >= 400:
                                continue
                            okurls.append(url)
                        except (requests.exceptions.ConnectionError,
                                requests.exceptions.ReadTimeout,
                                requests.exceptions.TooManyRedirects) as err:
                            self.logger.debug(err)
                            continue

                    if len(okurls) == 0:
                        continue

                    doc["urls"] = okurls
                    output.write(json.dumps(doc))
                    output.write("\n")

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='ldj.gz'), format=Gzip)



#!/usr/bin/env python3
# coding: utf-8

import io
import re
import sys

import xmltodict
import marcx


#Article</dc:type>
#Charter</dc:type>
#Fragment</dc:type>
#Letter</dc:type>
#Manuscript</dc:type>
#Map</dc:type>
#Monograph</dc:type>
#Multivolume_work</dc:
#Periodical</dc:type>
#Series</dc:type>
#Text</dc:type>
#Volume</dc:type>


inputfilename = "107_input.xml" 
outputfilename = "107_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

inputfile = open(inputfilename, "rb")
outputfile = open(outputfilename, "wb")

xmlfile = inputfile.read()
xmlrecords = xmltodict.parse(xmlfile)

for xmlrecord in xmlrecords["Records"]["Record"]:
    
    if not xmlrecord["metadata"]["oai_dc:dc"].get("dc:title"):
        continue       
    
    marcrecord = marcx.Record(force_utf8=True)

    # Leader
    marcrecord.leader = "     cam  22        4500"
    
    # Identifier
    f001 = xmlrecord["header"]["identifier"]
    regexp = re.match("oai:digi.ub.uni-heidelberg.de:(\d+)", f001)
    if regexp:
        f001 = regexp.group(1)
        marcrecord.add("001", data="finc-107-" + f001)
    else:
        print("Der Identifier konnte nicht zerlegt werden: " + f001)

    # 007
    marcrecord.add("007", data="cr")

    # Sprache
    language = xmlrecord["metadata"]["oai_dc:dc"]["dc:language"]   
    marcrecord.add("008", data="130227uu20uuuuuuxx uuup%s  c" % language)
    marcrecord.add("041", a=language)   

    # Verfasser
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:creator"):
        f100a = xmlrecord["metadata"]["oai_dc:dc"]["dc:creator"]
        marcrecord.add("100", a=f100a)

    # Titel   
    f245 = xmlrecord["metadata"]["oai_dc:dc"]["dc:title"]   
    marcrecord.add("245", a=f245)

    # Erscheinungsvermerk
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:date"):
        f260c = xmlrecord["metadata"]["oai_dc:dc"]["dc:date"]
        publisher = ["b", "Hochschule Mittweida,", "c", f260c]
        marcrecord.add("260", subfields=publisher)

    # Schlagwörter   
    if xmlrecord["metadata"]["oai_dc:dc"].get("dc:subject"):
        f689a = xmlrecord["metadata"]["oai_dc:dc"]["dc:subject"]   
        if isinstance(f689a, list):
            for subject in f689a:
                if ";" not in subject:
                    marcrecord.add("689", a=subject)
        else:          
            f689a = f689a.split(" , ")
            if len(f689a) > 1:
                for subject in f689a:
                    if ";" not in subject:
                        marcrecord.add("689", a=subject)

    # Link zu Datensatz und Ressource  
    f856u = xmlrecord["metadata"]["oai_dc:dc"]["dc:identifier"]
    if len(f856u) == 2:
        marcrecord.add("856", q="text/html", _3="Link zum Datensatz", u=f856u[0])  
        marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u[1])
    else:
        print("Die URLs weichen vom üblichen Schema ab: " + f001)

    # Medientyp
    marcrecord.add("935", b="cofz")
   
    # Kollektion
    # noch Reihenfolge herstellen
    marcrecord.add("980", a=f001, b="107", c="sid-107-col-heidelberg")

    
    outputfile.write(marcrecord.as_marc())


inputfile.close()
outputfile.close()


# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2018 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Refs #13842.

Mediathekviewweb (https://mediathekviewweb.de/) is a German media archival
project.  Its archive can currently be found at
https://verteiler.mediathekviewweb.de/archiv/.

Daily file file exports, xz.

    $ xz -l 2018-08-28-filme.xz 
    Strms  Blocks   Compressed Uncompressed  Ratio  Check   Filename
        1       1     23.8 MiB    147.2 MiB  0.162  CRC64   2018-08-28-filme.xz

The file 2018-08-28-filme can be parsed as JSON, but it is not JSON (cf.
https://git.io/fATFM, https://git.io/fATNI).

The first archive entry dates back to 2015-03-08,
https://verteiler.mediathekviewweb.de/archiv/2015/03/2015-03-08-filme.xz.

Config
------

[mediathekviewweb]

archive = https://verteiler.mediathekviewweb.de/archiv

"""

import datetime
import luigi
from gluish.utils import shellout
from siskin.task import DefaultTask


def yesterday():
    """
    Refs: #13842, #note-30.
    """
    return datetime.date.today() - datetime.timedelta(1)

class MVWTask(DefaultTask):
    """
    Media view web.
    """
    TAG = '169'


class MVWDownload(MVWTask):
    """
    Download file list for a given date.
    """
    date = luigi.DateParameter(default=yesterday())

    def run(self):
        url = "%s/%04d/%02d/%s-filme.xz" % (self.config.get("mediathekviewweb", "archive"),
                                        self.date.year, self.date.month, self.date)
        output = shellout("""curl --fail "{url}" > {output}""", url=url)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="xz"))


class MVWMARC(MVWTask):
    """
    Convert to a MARC via script.

    Note: The MARC file for today will request the download for yesterday.

        $ taskdeps MVWMARC
          └─ MVWMARC(date=2018-08-29)
                └─ MVWDownload(date=2018-08-28)

    """
    date = luigi.DateParameter(default=datetime.date.today())

    def requires(self):
        return MVWDownload(date=self.date - datetime.timedelta(1))

    def run(self):
        output = shellout("python {script} <(xz -dc {input}) {output}",
                          script=self.assets("169/169_marcbinary.py"),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext="fincmarc.mrc"))




# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
IMSLP, refs #1240, #13055.

Config
------

[imslp]

listings-url = http://example.com/export/
legacy-mapping = /path/to/json.file
"""

import datetime
import json
import os
import shutil
import tempfile

import luigi

from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.decorator import deprecated
from siskin.task import DefaultTask
from siskin.utils import scrape_html_listing
from siskin.conversions import imslp_tarball_to_marc

class IMSLPTask(DefaultTask):
    """ Base task for IMSLP. """
    TAG = "15"

    def closest(self):
        """ XXX: adjust. """
        return datetime.date(2017, 12, 25)

    def latest_link(self):
        """
        Find the lastest link on the listing.
        """
        listings_url = self.config.get("imslp", "listings-url")
        links = sorted([link for link in scrape_html_listing(listings_url)
                        if "imslpOut_" in link])

        self.logger.debug("found %s links on IMSLP download site", len(links))

        if len(links) == 0:
            raise ValueError("could not find any suitable links: %s", listings_url)

        return links[-1]

class IMSLPDownloadDeprecated(IMSLPTask):
    """ Download raw data. Should be a single URL pointing to a tar.gz. """

    date = ClosestDateParameter(default=datetime.date.today())

    @deprecated
    def run(self):
        output = shellout("""wget -O {output} {url}""",
                          url=self.config.get('imslp', 'url'))
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='tar.gz'))

class IMSLPDownload(IMSLPTask):
    """
    Download raw data. Actually keeps the original filename. Should be a tar.gz.
    """

    date = luigi.DateParameter(default=datetime.date.today())

    def run(self):
        """
        Assume, links are sortable, the last item should contain the latest link.
        XXX: Do not download, if not newer than last downloaded file.
        """
        most_recent = self.latest_link()
        self.logger.debug("most recent package seem to be: %s", most_recent)
        output = shellout(""" wget -O {output} "{url}" """, url=most_recent)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        filename = os.path.basename(self.latest_link())
        dst = os.path.join(self.taskdir(), filename)
        return luigi.LocalTarget(path=dst)

class IMSLPLegacyMapping(IMSLPTask, luigi.ExternalTask):
    """

    Path to JSON file mapping record id to viaf id and worktitle ("Werktitel").

    This is a fixed file.

        {
            "fantasiano10mudarraalonso": {
                "title": "Fantasie No.10",
                "viaf": "(VIAF)100203803"
            },
            ...
        }
    """

    def output(self):
        return luigi.LocalTarget(path=self.config.get('imslp', 'legacy-mapping'))

class IMSLPConvert(IMSLPTask):
    """
    Take a current version of the data plus legacy mapping and convert.

    WIP, refs #12288, refs #13055.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return {
            'legacy-mapping': IMSLPLegacyMapping(),
            'data': IMSLPDownload(date=self.date),
        }

    def run(self):
        """
        Load mapping, convert, write.
        """
        with self.input().get("legacy-mapping").open() as handle:
            mapping = json.load(handle)

        output = imslp_tarball_to_marc(self.input().get("data").path,
                                       legacy_mapping=mapping)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        filename = os.path.basename(self.latest_link())
        dst = os.path.join(self.taskdir(), filename.replace("tar.gz", "fincmarc.mrc"))
        return luigi.LocalTarget(path=dst)

class IMSLPConvertDeprecated(IMSLPTask):
    """
    Extract and transform.

    TODO, refs #13055 -- see IMSLPDownloadNext and IMSLPConvertNext and IMSLPLegacyMapping.

    File "/usr/lib/python2.7/site-packages/siskin/assets/15/15_marcbinary.py", line 165, in <module>
        record = record["document"]
    KeyError: 'document'
    """

    date = ClosestDateParameter(default=datetime.date.today())
    debug = luigi.BoolParameter(description='do not delete temporary folder')

    def requires(self):
        return IMSLPDownloadDeprecated(date=self.date)

    @deprecated
    def run(self):
        tempdir = tempfile.mkdtemp(prefix='siskin-')
        shellout("tar -xzf {archive} -C {tempdir}",
                 archive=self.input().path, tempdir=tempdir)
        output = shellout("python {script} {tempdir} {output}",
                          script=self.assets('15/15_marcbinary.py'), tempdir=tempdir)
        if not self.debug:
            shutil.rmtree(tempdir)
        else:
            self.logger.debug("not deleting temporary folder at %s", tempdir)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
Persee, refs #3133, #11349.
"""

import datetime

import luigi
import marcx
import pymarc
import ujson as json
from gluish.format import TSV
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.sources.amsl import AMSLService
from siskin.task import DefaultTask


class PerseeTask(DefaultTask):
    """ Base task. """
    TAG = '39'

    def closest(self):
        return monthly(self.date)


class PerseeCombined(PerseeTask):
    """
    Harvest and combine.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        """
        Explicit from, because earliestDate might be off, refs #11349
        (#note-65). In doubt, delete the cache via `rm -rf $(metha-sync -dir
        http://oai.persee.fr/oai)`.
        """
        output = shellout("""
            metha-sync -from 2000-01-01 http://oai.persee.fr/oai &&
            metha-cat http://oai.persee.fr/oai | pigz -c > {output}
        """)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml.gz'))


class PerseeMARC(PerseeTask):
    """
    Custom convert to MARC.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return PerseeCombined(date=self.date)

    def run(self):
        output = shellout("""python {script} <(unpigz -c {input}) {output}""",
                          script=self.assets('39/39_marcbinary.py'),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))

    def on_success(self):
        """
        Signal for manual update. This is decoupled from the task output,
        because it might be deleted, when imported.
        """
        self.output().copy(self.path(ext='fincmarc.mrc.import'))


class PerseeMARCIssue11349(PerseeTask):
    """
    Attach a special note to the item, namely another collection name, that,
    when processed with solrmarc, yield some desired result, refs #11349.
    TODO(miku): This is a temporary workaround.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return {
            'marc': PerseeMARC(date=self.date),
            'amsl': AMSLService(date=self.date),
        }

    def run(self):
        with self.input().get('amsl').open() as handle:
            amsl = json.load(handle)

        for item in amsl:
            if item["sourceID"] == "39" and item["ISIL"] == "DE-15-FID":
                link = item["linkToHoldingsFile"]
                break
        else:
            raise ValueError("cannot find link to DE-15-FID holdings file for SID 39")

        output = shellout("""curl --fail "{url}" > {output} """, url=link)

        issns = set()

        with luigi.LocalTarget(output, format=TSV).open() as handle:
            for row in handle.iter_tsv(cols=('issn',)):
                issns.add(row.issn.strip())

        updated = 0
        with self.input().get('marc').open('rb') as handle:
            with self.output().open('wb') as output:
                reader = pymarc.MARCReader(handle, to_unicode=True, force_utf8=True)
                writer = pymarc.MARCWriter(output)
                for i, record in enumerate(reader):
                    record = marcx.Record.from_record(record)

                    if not record["022"]:
                        writer.write(record)
                        continue

                    if "a" in record["022"] and record["022"]["a"].strip() in issns:
                        self.logger.debug("updating: %s", record["001"].data)
                        record.add("980", c="persee.fr (adlr)")
                        updated += 1
                    writer.write(record)

        self.logger.debug("updated %s records", updated)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))



#!/usr/bin/env python3
# coding: utf-8

import io
import sys
import re
import json

import marcx


formatmap = {
    "Buch": "Buch",
    "Konferenzbericht": "Buch",
    "Literaturzusammenstellung": "Buch",
    "Manuskript": "Buch",
    "Proceedings": "Buch",
    "Prospektmaterial": "Buch",
    "Seminarvortrag": "Buch",
    "Tagungsband": "Buch",
    "Verzeichnis": "Buch",
    "Wörterbuch": "Buch",
    "Bericht": "Buch",
    "Tagungsbericht": "Buch",
    "Diplomarbeit": "Hochschulschrift",
    "Dissertation": "Hochschulschrift",
    "Zeitschrift": "Artikel", # Zeitschriften sind hier eigentlich Zeitschriftenaufsätze
    "Artikel": "Artikel",
    "Aufsatz": "Artikel",
    "Aufsatz Kinderzeitschrift": "Artikel",
    "Rezension": "Artikel",
    "Verweisung": "Artikel",
    "Zeitschriftenartikel": "Artikel",
    "Zeitungsartikel": "Artikel",
    "Karte": "Karte",
    "Software": "Software",
    "CD-ROM": "Datenträger",
    "Datenbank": "Datenträger"
}


def getSubjects(tag):
    return jsonrecord[tag]


inputfilename = "131_input.json"
outputfilename = "131_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

with open(inputfilename, "r") as inputfile:
    jsonrecords = json.load(inputfile)

outputfile = io.open(outputfilename, "wb")

for jsonrecord in jsonrecords:

    if not jsonrecord["ID"] or not jsonrecord["TITLE"]:
        continue

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False
    format = jsonrecord["FORMAT"]
   
    if formatmap[format] == "Buch":
       leader = "     nam  22        4500"
       f007 = "tu"
       f008 = ""
       f935b = "druck"
       f935c = ""
    elif formatmap[format] == "Hochschulschrift":
        leader = "     nam  22        4500"
        f007 = "tu"
        f008 = ""
        f935b = ""
        f935c = "hs"    
    elif formatmap[format] == "Artikel":
        leader = "     cab  22        4500"
        f007 = "tu"
        f008 = ""
        f935b = "SAXB"
        f935c = "druck"
    elif formatmap[format] == "Karte":
        leader = "     cem  22        4500"
        f007 = "au"
        f008 = ""
        f935b = "druck"
        f935c = "kart" 
    elif formatmap[format] == "Software":
        leader = "     cgm  22        4500"
        f007 = "co"
        f008 = ""
        f935b = "crom"
        f935c = "lo"
    elif formatmap[format] == "Datenträger":
        leader = "     cgm  22        4500"
        f007 = "v"
        f008 = ""
        f935b = "soerd"
        f935c = ""    
    else:
        print("Format %s ist nicht in der Mapping-Tabelle enthalten" % format)

    # Leader
    marcrecord.leader = leader

    #Identifikator
    f001 = jsonrecord["ID"]
    f001 = f001.split("_")
    f001 = f001[1] 
    marcrecord.add("001", data="finc-131-" + f001)

    # Format (007)
    marcrecord.add("007", data=f007)

    # Erscheinungsweise (008)
    marcrecord.add("008", data=f008)
    
    # ISBN
    f020a = jsonrecord["ISBN"]
    marcrecord.add("020", a=f020a)

    # 1. Schöpfer
    authors = jsonrecord["AUTHOR"]
    if authors != "N.N." and authors != "Autorenteam":
        authors = authors.split(";")
        f100a = authors[0]     
        marcrecord.add("100", a=f100a)     
    else:
        authors = ""

    # Haupttitel
    f245a = jsonrecord["TITLE"]
    marcrecord.add("245", a=f245a)
   
    # Erscheinungsvermerk   
    f260c = jsonrecord["YEAR"]
    marcrecord.add("260", c=f260c)

    # Seitenzahl    
    f300 = jsonrecord["VOL_ISSUE"]
    f300 = f300.split("/")
    if len(f300) == 3:     
        pages = f300[2]      
        regexp = re.search("\D1-(\d+)", pages)
        if regexp:
            f300a = regexp.group(1)
            f300a = f300a + " S."
        else:
            f300a = ""
        marcrecord.add("300", a=f300a)

    # Hochschulvermerk
    if formatmap[format] == "Hochschulschrift":
        f502a = jsonrecord["CONT_TITLE"]
        marcrecord.add("502", a=f502a)
   
    # Format (935bc)  
    marcrecord.add("935", b=f935b, c=f935c)

    # Schlagwörter
    keywords = getSubjects("TOPIC_DETAILED")  # ist immer eine Liste
    if "Buch" in keywords:
        keywords.remove("Buch")
    
    if "Zeitschrift" in keywords:
       keywords.remove("Zeitschrift")

    substances = getSubjects("SUBSTANCE")
    if "/" in substances:
        substances = substances.split("/")
    else:
        substances = substances.split(",")
    
    elements = getSubjects("ELEMENTS")
    if "/" in elements:
        elements = elements.split("/")
    else:
        elements = elements.split(",")
        
    for keyword in keywords:
        marcrecord.add("650", a=keyword.strip())
    
    if len(substances) == 2 and substances[1][0].islower():            
        substance = ", ".join(substances)
        marcrecord.add("650", a=substance.strip())
    else:
        for substance in substances:
            if substance not in keyword:
                marcrecord.add("650", a=substance.strip())
    
    if len(elements) == 2 and elements[1][0].islower():         
            element = ", ".join(elements)
            marcrecord.add("650", a=element.strip())
    else:
        for element in elements:
            if element not in keyword:
                marcrecord.add("650", a=element.strip())

    if format not in keywords and format != "Buch" and format != "Zeitschrift":
        marcrecord.add("650", a=format)

    # weitere Schöpfer
    if len(authors) > 1:
        for f700a in authors[1:]:
            f700a = f700a.strip()           
            if f700a != "u.a.":
                marcrecord.add("700", a=f700a)

    # Quelle
    f773t = jsonrecord["CONT_TITLE"] # wenn kein vollständiges f773g, ist f773t meist nur "Buch" oder "Beitrag"    
    f773 = jsonrecord["VOL_ISSUE"]
    if f773 == "" and f773t == "" and format == "Zeitschrift":       
        print("Der folgende Aufsatz hat keine übergordnete Zeitschrift:" + f001)
    f773 = f773.split("/")
    if len(f773) == 3:
        volume = f773[0]
        issue = f773[1]
        pages = f773[2]
        year = jsonrecord["YEAR"]
        f773g = "%s(%s) Heft %s, S. %s" % (volume, year, issue, pages)        
    else:
        f773g = f773[0] # hier steht viel Murks, eventuell f773g = ""
    if formatmap[format] == "Artikel":
        marcrecord.add("773", t=f773t, g=f773g)

    # Kollektion
    marcrecord.add("980", a=f001, b="131", c="GDMB")

    outputfile.write(marcrecord.as_marc())

outputfile.close()


#!/usr/bin/env python
# coding: utf-8

"""
Archive.org to MARC. Allows variable number of inputs.

    $ python 153_marcbinary.py OUTPUT INPUT [INPUT, ...]

Post-process link check via [clinker](https://github.com/miku/clinker), 17min
for about 12000 records.

    $ time marctojson -p $(taskoutput ArchiveMARC) | \
        jq -c -r '.["856"][] | .["u"][] | {"url": .}' | \
        clinker -w 16 > 153.report.ndj

    $ jq .status 153.report.ndj | sort | uniq -c | sort -nr                                                                                                      │·
    12801 200

"""

from __future__ import print_function

import base64
import glob
import io
import json
import os
import re
import sys

import marcx
import xmltodict


colormap = {
   u"black and white": u"Schwarzweiß",
   u"black & white": u"Schwarzweiß",
   u"B&W": u"Schwarzweiß",
   u"color": u"Farbe",
   u"b&w": u"Schwarzweiß",
   u"Color": u"Farbe",
   u"Mix": u"Schwarzweiß / Farbe",
   u"B&W (tinded)": u"Schwarzweiß (koloriert)",
   u"b/w": u"Schwarzweiß",
   u"c": u"Farbe",
   u"tinted": u"Schwarzweiß (koloriert)",
   u"bw": u"Schwarzweiß",
   u"C": u"Farbe",
   u"B&W/C": u"Schwarzweiß / Farbe",
   u"tinted B&W": u"Schwarzweiß (koloriert)",
   u"B&W (tinted)": u"Schwarzweiß (koloriert)",
   u"sepia": u"Farbe",
   u"B/W": u"Farbe",
   u"color/B&W": u"Schwarzweiß / Farbe",
   u"b&w/color": u"Schwarzweiß / Farbe",
   u"Black and White": u"Schwarzweiß",
   u"B&w": u"Schwarzweiß"
}

soundmap = {
    u"black and white": u"Schwarzweiß",
    u"sound": u"Ton",
    u"silent": u"Stumm",
    u"No": u"Stumm",
    u"Yes": u"Ton",
    u"Sd": u"Ton",
    u"si": u"Stumm",
    u"Si": u"Stumm",
    u"silen": u"Stumm",
    u"sd": u"Ton",
    u"SD": u"Ton"
}


def get_field(tag):
    try:
        return jsonrecord[tag]
    except:
        return ""

def remove_tags(field):
    if isinstance(field, str):
        return re.sub("\<.*?\>", "", field)
    else:
        return ""


# input_directory = "153"
outputfilename = "153_output.mrc"
input_filenames = glob.glob('153/*.ldj')

if len(sys.argv) > 2:
    outputfilename, input_filenames = sys.argv[1], sys.argv[2:]

outputfile = open(outputfilename, "wb")

for filepath in input_filenames:
    filename = os.path.basename(filepath)
    inputfile = io.open(filepath, "r", encoding="utf-8")

    for line in inputfile:

        jsonobject = json.loads(line)

        try:
            jsonrecord = jsonobject["metadata"]
        except:
            print(filename, file=sys.stderr)
            continue

        marcrecord = marcx.Record(force_utf8=True)

        # Leader
        marcrecord.leader = "     cam  22        4500"

        # Identifier
        try:
            f001 = jsonrecord["identifier"]
        except:
            continue

        f001 = f001.encode("utf-8")
        f001 = base64.b64encode(f001)
        f001 = f001.decode("ascii")
        f001 = f001.rstrip("=")
        marcrecord.add("001", data="finc-153-" + f001)

        # Format
        marcrecord.add("007", data="cr")

        # Urheber
        f110a = get_field("creator")
        if f110a != "" and f110a != "Unknown":
            marcrecord.add("110", a=f110a)

        # Hauptitel
        f245a = get_field("title")
        if "delete" in f245a or "Delete" in f245a:
            continue
        marcrecord.add("245", a=f245a)

        # gestattet leere Felder, solange der Titel und der Identifier vorhanden sind
        marcrecord.strict = False

        # Filmstudio
        f260b = get_field("publisher")

        # Erscheinungsjahr
        f260c = get_field("date")
        if f260c != "":
            regexp = re.search("(\d\d\d\d)", f260c)
            if regexp:
                f260c = regexp.group(1)

        # Verlag
        if f260b != "" and f260b == f110a:
            f260b = ""  # verhindert, dass Körperschaft und Verlag nebeneinander im Katalog angezeigt werden, wenn beide identisch sind

        if f260b != "" and f260c != "":
            f260b = f260b + ", "  # ergänzt das Trennzeichen zwischen Produktionsfirma und Jahr, wenn beides vorhanden
        publisher = ["b", f260b, "c", f260c]
        marcrecord.add("260", subfields=publisher)

        # Spielzeit
        runtime = get_field("runtime")

        # Bild
        color_old = get_field("color")
        if color_old != "":
            color = colormap.get(color_old, "")
        else:
            color = ""
        if color == "" and color_old != "":
            print("Die Farbe %s wurde in der Colormap nicht gefunden" % color_old, file=sys.stderr)

        # Ton
        sound_old = get_field("sound")
        if sound_old != "":
            sound = soundmap.get(sound_old, "")
        else:
            sound = ""
        if sound == "" and sound_old != "":
            print("Der Sound %s wurde in der Soundmap nicht gefunden" % sound_old, file=sys.stderr)

        if color != "" and sound != "":
            f300b = color + " + " + sound
        elif color != "":
            f300b = color
        elif sound != "":
            f300b = sound
        else:
            f300b = ""

        if runtime != "":
            runtime = runtime.lstrip("00:")
            runtime = runtime.lstrip("0:")
            runtime = " (" + runtime
            if "min." not in runtime:
                runtime = runtime + " min.)"
            else:
                runtime = runtime + ")"
            f300a = f300b + runtime
        else:
            f300a = f300b

        marcrecord.add("300", a=f300a)

        # Spielzeit (extra MARC-Feld)
        f306a = get_field("runtime")
        if f306a != "":
            f306a = f306a.lstrip("00:")
            f306a = f306a.lstrip("0:")
            if "min" not in f306a:
                f306a = f306a + " min."
        marcrecord.add("306", a=f306a)

        # Soundformat (extra MARC-Feld)
        marcrecord.add("344", a=sound)

        # Bildformat (extra -MARC-Feld)
        marcrecord.add("346", a=color)

        # Annotation
        f520a = get_field("description")
        f520a = remove_tags(f520a)
        marcrecord.add("520", a=f520a)

        # Schlagwörter
        subjects = jsonrecord.get("subject", "")
        if subjects != "":
            if isinstance(subjects, list):
                for subject in subjects:
                    subject = subject.title()
                    marcrecord.add("650", a=subject)
            else:
                if subjects != "need keyword":
                    subject = subjects.title()
                    subjects = subject.split(";")
                    if isinstance(subjects, list):
                        for subject in subjects:
                            subject = subject.title()
                            marcrecord.add("650", a=subject)
                    else:
                        marcrecord.add("650", a=subject)


        # hebt die Erlaubnis für leere Felder wieder auf
        marcrecord.strict = True

        # Link zur Ressource
        f856u = get_field("identifier")
        marcrecord.add("856", q="text/html", _3="Link zur Ressource", u="https://archive.org/details/" + f856u)

        # Medienform
        marcrecord.add("935", b="cofz", c="vide")

        # Kollektion
        if "prelinger" in jsonrecord["collection"]:
            f980c = "sid-153-prelinger"
        elif "classic_cartoons" in jsonrecord["collection"]:
            f980c = "sid-153-col-classiccartoons"
        elif "feature_films" in jsonrecord["collection"]:
            f980c = "sid-153-col-featurefilms"
        elif "more_animation" in jsonrecord["collection"]:
            f980c = "sid-153-col-moreanimation"
        elif "vintage_cartoons" in jsonrecord["collection"]:
            f980c = "sid-153-col-vintagecartoons"
        else:
            raise ValueError('collection mapping missing: %s', jsonrecord['collection'])

        marcrecord.add("980", a=f001, b="153", c=f980c)

        try:
            outputfile.write(marcrecord.as_marc())
        except UnicodeDecodeError as exc:
            raise ValueError("%s: %s" % (marcrecord["001"].value(), exc))

    inputfile.close()

outputfile.close()



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2019 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>
#

"""
VKFilm Duesseldorf, #8392, MAB shipments.

External Java app required (https://sourceforge.net/projects/dnb-conv-tools/):

    $ sha1sum Mab2Mabxml-1.9.9.jar
    b37c4663fe6e4dcc55e71253266516e417ec9c44  Mab2Mabxml-1.9.9.jar

Config
------

[vkfilmdus]

input = /path/to/file.zip
Mab2Mabxml.jar = /path/to/local/copy.jar

"""

import datetime
import os

import luigi

from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout
from siskin.task import DefaultTask


class VKFilmDusTask(DefaultTask):
    """
    Base task for VK FILM DUESSELDORF, refs #8392.
    """
    TAG = '142'

    def closest(self):
        """
        Cf. 8392#note-38, sha1sum 55b5e6c026c5f07eeaafd3a8f04e14d79e63f20b.
        """
        return datetime.date(2019, 2, 19)


class VKFilmDusConvert(VKFilmDusTask):
    """
    Convert from binary MAB to MABXML.
    """
    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        """
        Extract zip on the fly, convert with Mab2Mabxml.jar.
        """
        mabfile = shellout("unzip -p {input} > {output}",
                          input=self.config.get('vkfilmdus', 'input'), encoding='utf-8')
        output = shellout("java -jar {jarfile} -encin ISO-8859-1 -i {input} -o {output}",
                          jarfile=self.config.get("vkfilmdus", "Mab2Mabxml.jar"),
                          input=mabfile)
        luigi.LocalTarget(output).move(self.output().path)
        os.remove(mabfile)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='xml'))




#!/usr/bin/env python3
# coding: utf-8


# SID: 129
# Collection: geoscan
# refs: #9871

from __future__ import print_function

import io
import re
import sys
import json
import marcx
import pymarc
import feedparser


langmap = {
    "English": "eng",
    "French": "fre",
    "Russian": "rus",
    "Greek": "gre"
}


def get_field(tag):  
    try:
        return record[tag]
    except:
        return ""


inputfilename, outputfilename = "129_input.xml", "129_output.mrc"

if len(sys.argv) == 3:
    inputfilename, outputfilename = sys.argv[1:]

outputfile = io.open(outputfilename, "wb")

records = feedparser.parse(inputfilename)

for record in records.entries:

    marcrecord = marcx.Record(force_utf8=True)
    marcrecord.strict = False

    # Leader
    marcrecord.leader = "     naa  22        4500"

    # Identifikator
    f001 = get_field("guid")
    marcrecord.add("001", data="finc-129-" + f001)
    
    # Format
    marcrecord.add("007", data="cr")

    # Sprache
    languages = get_field("geoscan_language")
    if ";" in languages:
        languages = languages.split("; ")
        lang = []
        for language in languages:
            f041a = langmap.get(language, "")
            if f041a != "":
                lang.append("a")
                lang.append(f041a)
                marcrecord.add("041", subfields=lang)                
            else:                
                print(u"Die folgende Sprache fehlt in der Langmap: %s" % language, file=sys.stderr)
    else:
        f041a = langmap.get(languages, "")
        marcrecord.add("041", a=f041a)

    # 1. Urheber
    f100a = get_field("geoscan_author")
    if ";" in f100a:
        f100a = f100a.split("; ")
        f100a = f100a[0]
    marcrecord.add("100", a=f100a)

    # Haupttitel    
    f245a = get_field("title")    
    marcrecord.add("245", a=f245a)

    # Verlag
    f260a = get_field("geoscan_publisher")
    description = get_field("description")
    regexp1 = re.search("(\d\d\d\d),\s(.*?),\s(https.*)", description)
    regexp2 = re.search("(\d\d\d\d)\.,\s(https.*)", description)
    if regexp1:
        f260c, f300a, link = regexp1.groups()
        f260c = ", " + f260c
    elif regexp2:
        f260c, link = regexp2.groups()
        f260c = ", " + f260c
        f300a = ""
    else:
        print(u"Der folgende String konnte nicht mittels regulärer Ausdrücke zerlegt werden: %s" % description, file=sys.stderr)
        f260c = ""
        f300a = ""
        link = ""
    marcrecord.add("260", a=f260a, c=f260c)

    # Seitenzahl
    marcrecord.add("300", a=f300a)

    # Anzahl der Karten
    f500a = get_field("geoscan_maps")
    marcrecord.add("500", a=f500a)

    # Angaben zur Karte
    f500a = get_field("geoscan_mapinfo")
    marcrecord.add("500", a=f500a)

    # Kurzreferat
    f520a = get_field("geoscan_abstract")
    marcrecord.add("520", a=f520a)

    # Schlagwörter
    subject = get_field("geoscan_province")
    if ";" in subject:
        subjects = subject.split("; ")
        for subject in subjects:
            marcrecord.add("650", a=subject)
    else:
        marcrecord.add("650", a=subject)
    
    subject = get_field("geoscan_area")
    if ";" in subject:
        subjects = subject.split("; ")
        for subject in subjects:
            marcrecord.add("650", a=subject)
    else:
        marcrecord.add("650", a=subject)

    # weitere Urheber
    f700a = get_field("geoscan_author")
    if ";" in f700a:
        authors = f700a.split("; ")
        for f700a in authors[1:]:
            marcrecord.add("700", a=f700a)

    # URL
    if link != "":
        f856u = link
    else:
        f856u = get_field("link")
    marcrecord.add("856", q="text/html", _3="Link zur Ressource", u=f856u)

    # 980
    collections = ["a", f001, "b", "129", "c", "geoscan"]
    marcrecord.add("980", subfields=collections)

    outputfile.write(marcrecord.as_marc())

outputfile.close()



# coding: utf-8
# pylint: disable=F0401,C0111,W0232,E1101,R0904,E1103,C0301

# Copyright 2017 by Leipzig University Library, http://ub.uni-leipzig.de
#                   The Finc Authors, http://finc.info
#                   Robert Schenk, <robert.schenk@uni-leipzig.de>
#                   Martin Czygan, <martin.czygan@uni-leipzig.de>
#
# This file is part of some open source application.
#
# Some open source application is free software: you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either
# version 3 of the License, or (at your option) any later version.
#
# Some open source application is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# @license GPL-3.0+ <http://spdx.org/licenses/GPL-3.0+>

"""
RISM, refs #4435.

[rism]

base-url = http://example.com/export

"""

import datetime
import os

import luigi
from gluish.intervals import monthly
from gluish.parameter import ClosestDateParameter
from gluish.utils import shellout

from siskin.task import DefaultTask


class RISMTask(DefaultTask):
    """ Base task for RISM. """
    TAG = "14"

    def closest(self):
        return monthly(self.date)


class RISMDownload(RISMTask):
    """
    Download raw data and prepare. Assume the URL (https://is.gd/ZTwzoT) of the
    complete dump does not change.

        Archive:  /tmp/gluish-S4wdR6
        Length      Date    Time    Name
        ---------  ---------- -----   ----
        3804391409  2017-03-23 14:46   rism_170316.xml <<<< We are only interested in this for now.
        52058685    2017-03-23 14:46   rism_ks.xml
        26080795    2017-03-23 14:46   rism_lit.xml
        191009578   2017-03-23 14:46   rism_pe.xml
        ---------                     -------
        4073540467                     4 files
    """

    date = ClosestDateParameter(default=datetime.date.today())

    def run(self):
        """
        Hack around the fact, that we do not know the exact filename
        (rism_170316.xml), but let's assume there is a pattern.
        """
        cleanup = set()
        url = "https://opac.rism.info/fileadmin/user_upload/lod/update/rismAllMARCXML.zip"

        output = shellout("""curl --fail "{url}" > {output} """, url=url)
        cleanup.add(output)

        output = shellout("""unzip -p {input} $(unzip -l {input} | grep -Eo "rism_[0-9]{{6,6}}.xml" | head -1) > {output}""", input=output)
        cleanup.add(output)

        output = shellout("yaz-marcdump -i marcxml -o marc {input} > {output}", input=output)
        luigi.LocalTarget(output).move(self.output().path)

        for file in cleanup:
            os.remove(file)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='mrc'))


class RISMMARC(RISMTask):
    """ Transform MARC. """

    date = ClosestDateParameter(default=datetime.date.today())

    def requires(self):
        return RISMDownload(date=self.date)

    def run(self):
        output = shellout("""python {script} {input} {output}""",
                          script=self.assets("14/14_marcbinary.py"),
                          input=self.input().path)
        luigi.LocalTarget(output).move(self.output().path)

    def output(self):
        return luigi.LocalTarget(path=self.path(ext='fincmarc.mrc'))
